--- utils-3.2.3.c	2010-09-21 17:05:31.000000000 +0200
+++ utils.c	2011-01-27 00:27:33.589499800 +0100
@@ -2876,7 +2876,8 @@
 	return OK;
         }
 
-
+#ifndef CYGWIN
+/* Standard -open_command_file- function
 
 /* creates external command file as a named pipe (FIFO) and opens it for reading (non-blocked mode) */
 int open_command_file(void){
@@ -2942,6 +2943,46 @@
 	return OK;
         }
 
+#else
+/* Cygwin-friendly open_command_file */
+
+/* Uses one-file-for-each-command approach -- no need for named pipe specific initializations ... */
+int open_command_file(void){
+	struct stat st;
+ 	int result=0;
+
+	/* we will use -rm- command to delete all commands files */
+	char rm_command [MAX_COMMAND_BUFFER];
+
+	/* if we're not checking external commands, don't do anything */
+	if(check_external_commands==FALSE)
+		return OK;
+
+	/* the command file was already created */
+	if(command_file_created==TRUE)
+		return OK;
+
+	/* initialize worker thread */
+	if(init_command_file_worker_thread()==ERROR){
+
+		logit(NSLOG_RUNTIME_ERROR,TRUE,"Error: Could not initialize command file worker thread.\n");
+
+		/* remove all files matching '<command_file>.*' */
+		(void) sprintf (rm_command, "rm -f %s.*", command_file);
+		system(rm_command);
+
+		return ERROR;
+	        }
+
+	/* set a flag to remember we already created the file */
+	command_file_created=TRUE;
+
+	return OK;
+        }
+#endif
+
+#ifndef CYGWIN
+/* Standard -close_command_file- function
 
 /* closes the external command file FIFO and deletes it */
 int close_command_file(void){
@@ -2963,7 +3004,32 @@
 	return OK;
         }
 
+#else
+/* Cygwin-friendly close_command_file */
+
+/* deletes all command files */
+int close_command_file(void){
 
+	char rm_command [MAX_COMMAND_BUFFER];
+
+	/* if we're not checking external commands, don't do anything */
+	if(check_external_commands==FALSE)
+		return OK;
+
+	/* the command file wasn't created or was already cleaned up */
+	if(command_file_created==FALSE)
+		return OK;
+
+	/* reset our flag */
+	command_file_created=FALSE;
+
+	(void) sprintf (rm_command, "rm -f %s.*", command_file);
+	system(rm_command);
+	
+	return OK;
+        }
+
+#endif
 
 
 /******************************************************************/
@@ -3893,7 +3959,8 @@
 	return;
         }
 
-
+#ifndef CYGWIN
+/* Standard -command_file_worker_thread- function */
 
 /* worker thread - artificially increases buffer of named pipe */
 void * command_file_worker_thread(void *arg){
@@ -4022,6 +4089,133 @@
 	return NULL;
         }
 
+#else
+/* Cygwin-friendly command_file_worker_thread */
+
+/* worker thread - scans external commands directory and submit commands */
+void * command_file_worker_thread(void *arg)
+{
+	char input_buffer[MAX_EXTERNAL_COMMAND_LENGTH];
+	int buffer_items=0;
+	int result=0;
+
+	char command_file_path[MAX_FILENAME_LENGTH];
+	char command_file_name[MAX_FILENAME_LENGTH];
+	char * command_path_end;
+	struct dirent **command_files;
+	int command_file_select();
+	int command_file_count;
+	extern  int alphasort();
+	int i;
+
+	/* specify cleanup routine */
+	pthread_cleanup_push(cleanup_command_file_worker_thread,NULL);
+
+	/* set cancellation info */
+	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
+	pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED,NULL);
+
+	while(1)
+	{
+		/* should we shutdown? */
+		pthread_testcancel();
+
+		/* For one-command-a-file approach, we scan command directory for files */
+		
+		/* Get directory part from command_file */
+		strncpy(command_file_path, command_file, MAX_FILENAME_LENGTH);
+		command_path_end = strrchr( (char *) command_file_path, '/');
+		*command_path_end = '\0';
+		
+		/* scan commands directory and create a file list sorted by name */
+		command_file_count = scandir(command_file_path, &command_files, 0, alphasort);
+		
+		/* loop if no files */
+ 		if(command_file_count==0) {
+			/* directory scan is quite expensive, wait some time then */
+			sleep(1);
+ 		
+			/* should we shutdown? */
+ 			pthread_testcancel();
+
+			sleep(1);
+ 			continue;
+		}
+			
+		/* check for errors */
+		if (command_file_count == -1)
+		{
+			write_to_log("command_file_worker_thread(): scandir()",logging_options,NULL);
+			continue;
+        }
+
+ 		/* should we shutdown? */
+ 		pthread_testcancel();
+
+		/* get number of items in the buffer */
+		pthread_mutex_lock(&external_command_buffer.buffer_lock);
+		buffer_items=external_command_buffer.items;
+		pthread_mutex_unlock(&external_command_buffer.buffer_lock);
+
+#ifdef DEBUG_CFWT
+		printf("(CFWT) BUFFER ITEMS: %d/%d\n",buffer_items,external_command_buffer_slots);
+#endif
+
+		/* Simple logic: Don't do anything if there is no room */
+		if((buffer_items + command_file_count) < external_command_buffer_slots)
+		{
+			/* Process commands */
+			for (i=0; i<command_file_count; i++)
+			{
+				/* skip '.' and '..' */
+				if ((strcmp(command_files[i]->d_name, ".") == 0) || (strcmp(command_files[i]->d_name, "..") == 0)) 
+					continue;
+			
+				/* form a complete path to the current command file */
+				(void *) sprintf (command_file_name, "%s/%s", command_file_path, command_files[i]->d_name);
+
+				/* open file, read it and delete it */
+				command_file_fp=fopen(command_file_name,"r");
+				fgets(input_buffer,(int)(sizeof(input_buffer)-1),command_file_fp);
+				fclose(command_file_fp);
+				unlink(command_file_name);
+
+#ifdef DEBUG_CFWT
+				printf("(CFWT) READ: %s",input_buffer);
+#endif
+				result = submit_external_command(input_buffer,&buffer_items);
+				
+				if (result == ERROR) {
+					write_to_log("command_file_worker_thread(): submit_external_command()",logging_options,NULL);
+					continue;
+				}
+
+#ifdef DEBUG_CFWT
+				printf("(CFWT) RES: %d, BUFFER_ITEMS: %d/%d\n",result,buffer_items,external_comand_buffer_slots);
+#endif
+
+				/* should we shutdown? */
+				pthread_testcancel();
+			}
+		}
+		
+		/* No need for directory entry array, give them back */
+		for (i=0; i<command_file_count; i++)
+		{
+			my_free(command_files[i]);
+		}
+		my_free(command_files);
+		
+	}
+
+	/* removes cleanup handler - this should never be reached */
+
+	pthread_cleanup_pop(0);
+	return NULL;
+	
+}
+
+#endif
 
 
 /* submits an external command for processing */
